@@ -645,7 +645,7 @@ gst_dvd_nav_src_update_streaminfo (GstDvdNavSrc * src)
 #endif
 }
 
-/**
+/*
  * Check for a new DVD domain area, and update the structure if
  * necessary.
  */
@@ -670,7 +670,7 @@ gst_dvd_nav_src_set_domain (GstDvdNavSrc * src)
   src->domain = domain;
 }
 
-/**
+/*
  * Check for a new highlighted area, and send an spu highlight event if
  * necessary.
  */
@@ -1183,7 +1183,7 @@ gst_dvd_nav_src_push_titlelang_event (GstDvdNavSrc * src)
 
   /* build event */
 
-  s = gst_structure_new ("application/x-gst-event",
+  s = gst_structure_new ("application/x-gst-dvd",
       "event", G_TYPE_STRING, "dvd-lang-codes", NULL);
   e = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
 
@@ -2188,4 +2188,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "dvdnav",
     "Access a DVD with navigation features using libdvdnav",
-    plugin_init, VERSION, "GPL", GST_PACKAGE, GST_ORIGIN)
+    plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
