@@ -1,6 +1,7 @@
-/* GStreamer
+/* GStreamer DVD title source
  * Copyright (C) 1999 Erik Walthinsen <omega@cse.ogi.edu>
  * Copyright (C) 2001 Billy Biggs <vektor@dumbterm.net>.
+ * Copyright (C) 2006 Tim-Philipp Müller <tim centricular net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,9 +33,7 @@
 
 #include "dvdreadsrc.h"
 
-/* #include <gst/gst-i18n-plugin.h> */
-/* FIXME: remove once GETTEXT_PACKAGE etc. is set */
-#define _(s) s
+#include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY_STATIC (gstgst_dvd_read_src_debug);
 #define GST_CAT_DEFAULT (gstgst_dvd_read_src_debug)
@@ -87,11 +86,20 @@ static void gst_dvd_read_src_get_property (GObject * object, guint prop_id,
 static GstEvent *gst_dvd_read_src_make_clut_change_event (GstDvdReadSrc * src,
     const guint * clut);
 static gboolean gst_dvd_read_src_get_size (GstDvdReadSrc * src, gint64 * size);
+static gboolean gst_dvd_read_src_do_seek (GstBaseSrc * src, GstSegment * s);
+static gint64 gst_dvd_read_src_convert_timecode (dvd_time_t * time);
+static gint gst_dvd_read_src_get_next_cell (GstDvdReadSrc * src,
+    pgc_t * pgc, gint cell);
+static GstClockTime gst_dvd_read_src_get_time_for_sector (GstDvdReadSrc * src,
+    guint sector);
+static gint gst_dvd_read_src_get_sector_from_time (GstDvdReadSrc * src,
+    GstClockTime ts);
 
 GST_BOILERPLATE_FULL (GstDvdReadSrc, gst_dvd_read_src, GstPushSrc,
-    GST_TYPE_PUSH_SRC, gst_dvd_read_src_do_init)
+    GST_TYPE_PUSH_SRC, gst_dvd_read_src_do_init);
 
-     static void gst_dvd_read_src_base_init (gpointer g_class)
+static void
+gst_dvd_read_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
@@ -129,9 +137,6 @@ gst_dvd_read_src_init (GstDvdReadSrc * src, GstDvdReadSrcClass * klass)
   src->uri_chapter = 1;
   src->uri_angle = 1;
 
-  src->seek_pend = FALSE;
-  src->flush_pend = FALSE;
-  src->seek_pend_fmt = GST_FORMAT_UNDEFINED;
   src->title_lang_event_pending = NULL;
   src->pending_clut_event = NULL;
 
@@ -140,6 +145,12 @@ gst_dvd_read_src_init (GstDvdReadSrc * src, GstDvdReadSrcClass * klass)
       gst_static_pad_template_get_caps (&srctemplate));
 }
 
+static gboolean
+gst_dvd_read_src_is_seekable (GstBaseSrc * src)
+{
+  return TRUE;
+}
+
 static void
 gst_dvd_read_src_class_init (GstDvdReadSrcClass * klass)
 {
@@ -168,6 +179,9 @@ gst_dvd_read_src_class_init (GstDvdReadSrcClass * klass)
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_dvd_read_src_stop);
   gstbasesrc_class->query = GST_DEBUG_FUNCPTR (gst_dvd_read_src_src_query);
   gstbasesrc_class->event = GST_DEBUG_FUNCPTR (gst_dvd_read_src_src_event);
+  gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR (gst_dvd_read_src_do_seek);
+  gstbasesrc_class->is_seekable =
+      GST_DEBUG_FUNCPTR (gst_dvd_read_src_is_seekable);
 
   gstpushsrc_class->create = GST_DEBUG_FUNCPTR (gst_dvd_read_src_create);
 }
@@ -181,24 +195,14 @@ gst_dvd_read_src_start (GstBaseSrc * basesrc)
 
   GST_DEBUG_OBJECT (src, "Opening DVD '%s'", src->location);
 
-  src->dvd = DVDOpen (src->location);
-  if (src->dvd == NULL) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        (_("Could not open DVD")),
-        ("DVDOpen(%s) failed: %s", src->location, g_strerror (errno)));
-    return FALSE;
-  }
+  if ((src->dvd = DVDOpen (src->location)) == NULL)
+    goto open_failed;
 
   /* Load the video manager to find out the information about the titles */
   GST_DEBUG_OBJECT (src, "Loading VMG info");
 
-  src->vmg_file = ifoOpen (src->dvd, 0);
-  if (!src->vmg_file) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        (_("Could not open DVD")),
-        ("ifoOpen() failed: %s", g_strerror (errno)));
-    return FALSE;
-  }
+  if (!(src->vmg_file = ifoOpen (src->dvd, 0)))
+    goto ifo_open_failed;
 
   src->tt_srpt = src->vmg_file->tt_srpt;
 
@@ -206,21 +210,45 @@ gst_dvd_read_src_start (GstBaseSrc * basesrc)
   src->chapter = src->uri_chapter - 1;
   src->angle = src->uri_angle - 1;
 
-  if (!gst_dvd_read_src_goto_title (src, src->title, src->angle)) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        (_("Could not open DVD title %d"), src->uri_title), (NULL));
-    return FALSE;
-  }
+  if (!gst_dvd_read_src_goto_title (src, src->title, src->angle))
+    goto title_open_failed;
 
-  if (!gst_dvd_read_src_goto_chapter (src, src->chapter)) {
-    GST_ERROR_OBJECT (src, "Failed to go to chapter %d of DVD title %d",
-        src->uri_chapter, src->uri_title);
-  }
+  if (!gst_dvd_read_src_goto_chapter (src, src->chapter))
+    goto chapter_open_failed;
 
   src->new_seek = FALSE;
   src->change_cell = TRUE;
 
   return TRUE;
+
+  /* ERRORS */
+open_failed:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (_("Could not open DVD")),
+        ("DVDOpen(%s) failed: %s", src->location, g_strerror (errno)));
+    return FALSE;
+  }
+ifo_open_failed:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (_("Could not open DVD")),
+        ("ifoOpen() failed: %s", g_strerror (errno)));
+    return FALSE;
+  }
+title_open_failed:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (_("Could not open DVD title %d"), src->uri_title), (NULL));
+    return FALSE;
+  }
+chapter_open_failed:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (_("Failed to go to chapter %d of DVD title %d"),
+            src->uri_chapter, src->uri_title), (NULL));
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -249,9 +277,8 @@ gst_dvd_read_src_stop (GstBaseSrc * basesrc)
   src->change_cell = FALSE;
   src->chapter = 0;
   src->title = 0;
-  src->flush_pend = FALSE;
-  src->seek_pend = FALSE;
-  src->seek_pend_fmt = GST_FORMAT_UNDEFINED;
+  src->need_newsegment = TRUE;
+  src->vts_tmapt = NULL;
   if (src->title_lang_event_pending) {
     gst_event_unref (src->title_lang_event_pending);
     src->title_lang_event_pending = NULL;
@@ -260,6 +287,10 @@ gst_dvd_read_src_stop (GstBaseSrc * basesrc)
     gst_event_unref (src->pending_clut_event);
     src->pending_clut_event = NULL;
   }
+  if (src->chapter_starts) {
+    g_free (src->chapter_starts);
+    src->chapter_starts = NULL;
+  }
 
   GST_LOG_OBJECT (src, "closed DVD");
 
@@ -324,7 +355,7 @@ gst_dvd_read_src_goto_chapter (GstDvdReadSrc * src, gint chapter)
   cur_title_get_chapter_bounds (src, chapter, &src->start_cell,
       &src->last_cell);
 
-  GST_LOG_OBJECT (src, "Opened chapter %d - cell %d-%d", chapter,
+  GST_LOG_OBJECT (src, "Opened chapter %d - cell %d-%d", chapter + 1,
       src->start_cell, src->last_cell);
 
   /* retrieve position */
@@ -357,6 +388,45 @@ gst_dvd_read_src_goto_chapter (GstDvdReadSrc * src, gint chapter)
   return TRUE;
 }
 
+static void
+gst_dvd_read_src_get_chapter_starts (GstDvdReadSrc * src)
+{
+  GstClockTime uptohere;
+  guint c;
+
+  g_free (src->chapter_starts);
+  src->chapter_starts = g_new (GstClockTime, src->num_chapters);
+
+  uptohere = (GstClockTime) 0;
+  for (c = 0; c < src->num_chapters; ++c) {
+    GstClockTime chapter_duration = 0;
+    gint cell_start, cell_end, cell;
+    gint pgn, pgc_id;
+    pgc_t *pgc;
+
+    cur_title_get_chapter_pgc (src, c, &pgn, &pgc_id, &pgc);
+    cur_title_get_chapter_bounds (src, c, &cell_start, &cell_end);
+
+    cell = cell_start;
+    while (cell < cell_end) {
+      dvd_time_t *cell_duration;
+
+      cell_duration = &pgc->cell_playback[cell].playback_time;
+      chapter_duration += gst_dvd_read_src_convert_timecode (cell_duration);
+      cell = gst_dvd_read_src_get_next_cell (src, pgc, cell);
+    }
+
+    src->chapter_starts[c] = uptohere;
+
+    GST_INFO_OBJECT (src, "[%02u] Chapter %02u starts at %" GST_TIME_FORMAT
+        ", dur = %" GST_TIME_FORMAT ", cells %d-%d", src->title + 1, c + 1,
+        GST_TIME_ARGS (uptohere), GST_TIME_ARGS (chapter_duration),
+        cell_start, cell_end);
+
+    uptohere += chapter_duration;
+  }
+}
+
 static gboolean
 gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
 {
@@ -364,53 +434,48 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
   gchar lang_code[3] = { '\0', '\0', '\0' }, *t;
   gint title_set_nr;
   gint num_titles;
-  gint num_angles;
   gint i;
 
   /* make sure our title number is valid */
   num_titles = src->tt_srpt->nr_of_srpts;
   GST_INFO_OBJECT (src, "There are %d titles on this DVD", num_titles);
-  if (title < 0 || title >= num_titles) {
-    GST_WARNING_OBJECT (src, "Invalid title %d (only %d available)",
-        title, num_titles);
-    return FALSE;
-  }
+  if (title < 0 || title >= num_titles)
+    goto invalid_title;
 
   src->num_chapters = src->tt_srpt->title[title].nr_of_ptts;
-  GST_INFO_OBJECT (src, "Title %d has %d chapters", title, src->num_chapters);
+  GST_INFO_OBJECT (src, "Title %d has %d chapters", title + 1,
+      src->num_chapters);
 
   /* make sure the angle number is valid for this title */
-  num_angles = src->tt_srpt->title[title].nr_of_angles;
-  GST_LOG_OBJECT (src, "Title %d has %d angles", title, num_angles);
-  if (angle < 0 || angle >= num_angles) {
+  src->num_angles = src->tt_srpt->title[title].nr_of_angles;
+  GST_LOG_OBJECT (src, "Title %d has %d angles", title + 1, src->num_angles);
+  if (angle < 0 || angle >= src->num_angles) {
     GST_WARNING_OBJECT (src, "Invalid angle %d (only %d available)",
-        angle, num_angles);
-    angle = CLAMP (angle, 0, num_angles - 1);
+        angle, src->num_angles);
+    angle = CLAMP (angle, 0, src->num_angles - 1);
   }
 
   /* load the VTS information for the title set our title is in */
   title_set_nr = src->tt_srpt->title[title].title_set_nr;
   src->vts_file = ifoOpen (src->dvd, title_set_nr);
-  if (src->vts_file == NULL) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        (_("Could not open DVD title %d"), title_set_nr),
-        ("ifoOpen(%d) failed: %s", title_set_nr, g_strerror (errno)));
-    return FALSE;
-  }
+  if (src->vts_file == NULL)
+    goto ifo_open_failed;
 
   src->ttn = src->tt_srpt->title[title].vts_ttn;
   src->vts_ptt_srpt = src->vts_file->vts_ptt_srpt;
 
-  /* we've got enough info, time to open the title set data */
-  src->dvd_title = DVDOpenFile (src->dvd, title_set_nr, DVD_READ_TITLE_VOBS);
-  if (src->dvd_title == NULL) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        (_("Could not open DVD title %d"), title_set_nr),
-        ("Can't open title VOBS (VTS_%02d_1.VOB)", title_set_nr));
-    return FALSE;
+  /* interactive title? */
+  if (src->num_chapters > 0 &&
+      src->vts_ptt_srpt->title[src->ttn - 1].ptt[0].pgn == 0) {
+    goto commands_only_pgc;
   }
 
-  GST_INFO_OBJECT (src, "Opened title %d, angle %d", title, angle);
+  /* we've got enough info, time to open the title set data */
+  src->dvd_title = DVDOpenFile (src->dvd, title_set_nr, DVD_READ_TITLE_VOBS);
+  if (src->dvd_title == NULL)
+    goto title_open_failed;
+
+  GST_INFO_OBJECT (src, "Opened title %d, angle %d", title + 1, angle);
   src->title = title;
   src->angle = angle;
 
@@ -443,7 +508,7 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
     }
 
     GST_INFO_OBJECT (src, "[%02d] Audio    %02d: lang='%s', format=%d",
-        src->title, i, lang_code, (gint) a->audio_format);
+        src->title + 1, i, lang_code, (gint) a->audio_format);
   }
 
   /* subtitle */
@@ -461,36 +526,167 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
     }
 
     GST_INFO_OBJECT (src, "[%02d] Subtitle %02d: lang='%s', format=%d",
-        src->title, i, lang_code);
+        src->title + 1, i, lang_code);
   }
 
   src->title_lang_event_pending =
       gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
 
+  /* dump seek tables */
+  src->vts_tmapt = src->vts_file->vts_tmapt;
+  if (src->vts_tmapt) {
+    gint i, j;
+
+    GST_LOG_OBJECT (src, "nr_of_tmaps = %d", src->vts_tmapt->nr_of_tmaps);
+    for (i = 0; i < src->vts_tmapt->nr_of_tmaps; ++i) {
+      GST_LOG_OBJECT (src, "======= Table %d ===================", i);
+      GST_LOG_OBJECT (src, "Offset relative to VTS_TMAPTI: %d",
+          src->vts_tmapt->tmap_offset[i]);
+      GST_LOG_OBJECT (src, "Time unit (seconds)          : %d",
+          src->vts_tmapt->tmap[i].tmu);
+      GST_LOG_OBJECT (src, "Number of entries            : %d",
+          src->vts_tmapt->tmap[i].nr_of_entries);
+      for (j = 0; j < src->vts_tmapt->tmap[i].nr_of_entries; j++) {
+        guint64 time;
+
+        time = src->vts_tmapt->tmap[i].tmu * (j + 1) * GST_SECOND;
+        GST_LOG_OBJECT (src, "Time: %" GST_TIME_FORMAT " VOBU "
+            "Sector: 0x%08x %s", GST_TIME_ARGS (time),
+            src->vts_tmapt->tmap[i].map_ent[j] & 0x7fffffff,
+            (src->vts_tmapt->tmap[i].map_ent[j] >> 31) ? "discontinuity" : "");
+      }
+    }
+  } else {
+    GST_WARNING_OBJECT (src, "no vts_tmapt - seeking will suck");
+  }
+
+  gst_dvd_read_src_get_chapter_starts (src);
+
   return TRUE;
+
+  /* ERRORS */
+invalid_title:
+  {
+    GST_WARNING_OBJECT (src, "Invalid title %d (only %d available)",
+        title, num_titles);
+    return FALSE;
+  }
+ifo_open_failed:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (_("Could not open DVD title %d"), title_set_nr),
+        ("ifoOpen(%d) failed: %s", title_set_nr, g_strerror (errno)));
+    return FALSE;
+  }
+title_open_failed:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (_("Could not open DVD title %d"), title_set_nr),
+        ("Can't open title VOBS (VTS_%02d_1.VOB)", title_set_nr));
+    return FALSE;
+  }
+commands_only_pgc:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (_("Could not open DVD title %d. Interactive titles are not supported "
+                "by this element"), title_set_nr),
+        ("Commands-only PGC, not supported, use dvdnavsrc"));
+    return FALSE;
+  }
 }
 
 /* FIXME: double-check this function, compare against original */
 static gint
-gst_dvd_read_src_get_next_cell_for (GstDvdReadSrc * src, gint cell)
+gst_dvd_read_src_get_next_cell (GstDvdReadSrc * src, pgc_t * pgc, gint cell)
 {
   /* Check if we're entering an angle block. */
-  if (src->cur_pgc->cell_playback[cell].block_type != BLOCK_TYPE_ANGLE_BLOCK)
+  if (pgc->cell_playback[cell].block_type != BLOCK_TYPE_ANGLE_BLOCK)
     return (cell + 1);
 
-  while (src->cur_pgc->cell_playback[cell].block_mode == BLOCK_MODE_LAST_CELL)
+  while (pgc->cell_playback[cell].block_mode == BLOCK_MODE_LAST_CELL)
     ++cell;
 
   return cell + 1;              /* really +1? (tpm) */
 }
 
-/* Returns true if the pack is a NAV pack.  This check is clearly insufficient,
- * and sometimes we incorrectly think that valid other packs are NAV packs.  I
- * need to make this stronger. */
+/* Returns true if the pack is a NAV pack */
 static gboolean
-gst_dvd_read_src_is_nav_pack (const guint8 * buffer)
+gst_dvd_read_src_is_nav_pack (const guint8 * data)
 {
-  return (buffer[41] == 0xbf && buffer[1027] == 0xbf);
+  if (GST_READ_UINT32_BE (data + 0x26) != 0x000001BF)
+    return FALSE;
+
+  /* Check that this is substream 0 (PCI) */
+  if (data[0x2c] != 0)
+    return FALSE;
+
+  if (GST_READ_UINT32_BE (data + 0x400) != 0x000001BF)
+    return FALSE;
+
+  /* Check that this is substream 1 (DSI) */
+  if (data[0x406] != 1)
+    return FALSE;
+
+  /* Check sizes of PCI and DSI packets */
+  if (GST_READ_UINT16_BE (data + 0x2a) != 0x03d4)
+    return FALSE;
+
+  if (GST_READ_UINT16_BE (data + 0x404) != 0x03fa)
+    return FALSE;
+
+  return TRUE;
+}
+
+/* find time for sector from index, returns NONE if there is no exact match */
+static GstClockTime
+gst_dvd_read_src_get_time_for_sector (GstDvdReadSrc * src, guint sector)
+{
+  gint i, j;
+
+  if (src->vts_tmapt == NULL || src->vts_tmapt->nr_of_tmaps == 0)
+    return GST_CLOCK_TIME_NONE;
+
+  for (i = 0; i < src->vts_tmapt->nr_of_tmaps; ++i) {
+    for (j = 0; j < src->vts_tmapt->tmap[i].nr_of_entries; ++j) {
+      if ((src->vts_tmapt->tmap[i].map_ent[j] & 0x7fffffff) == sector)
+        return src->vts_tmapt->tmap[i].tmu * (j + 1) * GST_SECOND;
+    }
+  }
+
+  if (sector == 0)
+    return (GstClockTime) 0;
+
+  return GST_CLOCK_TIME_NONE;
+}
+
+/* returns the sector in the index at (or before) the given time, or -1 */
+static gint
+gst_dvd_read_src_get_sector_from_time (GstDvdReadSrc * src, GstClockTime ts)
+{
+  gint sector, i, j;
+
+  if (src->vts_tmapt == NULL || src->vts_tmapt->nr_of_tmaps == 0)
+    return -1;
+
+  sector = 0;
+  for (i = 0; i < src->vts_tmapt->nr_of_tmaps; ++i) {
+    for (j = 0; j < src->vts_tmapt->tmap[i].nr_of_entries; ++j) {
+      GstClockTime entry_time;
+
+      entry_time = src->vts_tmapt->tmap[i].tmu * (j + 1) * GST_SECOND;
+      if (entry_time <= ts) {
+        sector = src->vts_tmapt->tmap[i].map_ent[j] & 0x7fffffff;
+      }
+      if (entry_time >= ts) {
+        return sector;
+      }
+    }
+  }
+
+  if (ts == 0)
+    return 0;
+
+  return -1;
 }
 
 typedef enum
@@ -519,12 +715,11 @@ again:
 
   if (src->cur_cell >= src->last_cell) {
     /* advance to next chapter */
-    if (src->chapter == (src->num_chapters - 1)) {
-      GST_INFO_OBJECT (src, "last chapter done - EOS");
-      return GST_DVD_READ_EOS;
-    }
+    if (src->chapter == (src->num_chapters - 1))
+      goto eos;
 
-    GST_INFO_OBJECT (src, "end of chapter %d, switch to next", src->chapter);
+    GST_INFO_OBJECT (src, "end of chapter %d, switch to next",
+        src->chapter + 1);
 
     ++src->chapter;
     gst_dvd_read_src_goto_chapter (src, src->chapter);
@@ -547,7 +742,8 @@ again:
       src->cur_cell += angle;
 
     /* calculate next cell */
-    src->next_cell = gst_dvd_read_src_get_next_cell_for (src, src->cur_cell);
+    src->next_cell =
+        gst_dvd_read_src_get_next_cell (src, src->cur_pgc, src->cur_cell);
 
     /* we loop until we're out of this cell */
     src->cur_pack = src->cur_pgc->cell_playback[src->cur_cell].first_sector;
@@ -564,10 +760,8 @@ again:
 nav_retry:
 
   len = DVDReadBlocks (src->dvd_title, src->cur_pack, 1, oneblock);
-  if (len == 0) {
-    GST_ERROR_OBJECT (src, "Read failed for block %d", src->cur_pack);
-    return GST_DVD_READ_ERROR;
-  }
+  if (len == 0)
+    goto read_error;
 
   if (!gst_dvd_read_src_is_nav_pack (oneblock)) {
     src->cur_pack++;
@@ -607,15 +801,13 @@ nav_retry:
   len = DVDReadBlocks (src->dvd_title, src->cur_pack, cur_output_size,
       GST_BUFFER_DATA (buf));
 
-  if (len != cur_output_size) {
-    GST_ERROR_OBJECT (src, "Read failed for %d blocks at %d",
-        cur_output_size, src->cur_pack);
-    gst_buffer_unref (buf);
-    return GST_DVD_READ_ERROR;
-  }
+  if (len != cur_output_size)
+    goto block_read_error;
 
   GST_BUFFER_SIZE (buf) = cur_output_size * DVD_VIDEO_LB_LEN;
   /* GST_BUFFER_OFFSET (buf) = priv->cur_pack * DVD_VIDEO_LB_LEN; */
+  GST_BUFFER_TIMESTAMP (buf) =
+      gst_dvd_read_src_get_time_for_sector (src, src->cur_pack);
 
   gst_buffer_set_caps (buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (src)));
 
@@ -624,7 +816,27 @@ nav_retry:
   src->cur_pack = next_vobu;
 
   GST_LOG_OBJECT (src, "Read %u sectors", cur_output_size);
+
   return GST_DVD_READ_OK;
+
+  /* ERRORS */
+eos:
+  {
+    GST_INFO_OBJECT (src, "last chapter done - EOS");
+    return GST_DVD_READ_EOS;
+  }
+read_error:
+  {
+    GST_ERROR_OBJECT (src, "Read failed for block %d", src->cur_pack);
+    return GST_DVD_READ_ERROR;
+  }
+block_read_error:
+  {
+    GST_ERROR_OBJECT (src, "Read failed for %d blocks at %d",
+        cur_output_size, src->cur_pack);
+    gst_buffer_unref (buf);
+    return GST_DVD_READ_ERROR;
+  }
 }
 
 static GstFlowReturn
@@ -638,34 +850,11 @@ gst_dvd_read_src_create (GstPushSrc * pushsrc, GstBuffer ** p_buf)
 
   srcpad = GST_BASE_SRC (src)->srcpad;
 
-  /* handle events, if any */
-  if (src->seek_pend) {
-    if (src->flush_pend) {
-      gst_pad_push_event (srcpad, gst_event_new_flush_start ());
-      gst_pad_push_event (srcpad, gst_event_new_flush_stop ());
-      src->flush_pend = FALSE;
-    }
-
-    if (src->seek_pend_fmt != GST_FORMAT_UNDEFINED) {
-      if (src->seek_pend_fmt == title_format) {
-        gst_dvd_read_src_goto_title (src, src->title, src->angle);
-      }
-      gst_dvd_read_src_goto_chapter (src, src->chapter);
-
-      src->seek_pend_fmt = GST_FORMAT_UNDEFINED;
-    } else {
-      if (!gst_dvd_read_src_goto_sector (src, src->angle)) {
-        GST_DEBUG_OBJECT (src, "seek to sector failed, going EOS");
-        gst_pad_push_event (srcpad, gst_event_new_eos ());
-        return GST_FLOW_UNEXPECTED;
-      }
-    }
-
+  if (src->need_newsegment) {
     gst_pad_push_event (srcpad,
         gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_BYTES,
             src->cur_pack * DVD_VIDEO_LB_LEN, -1, 0));
-
-    src->seek_pend = FALSE;
+    src->need_newsegment = FALSE;
   }
 
   if (src->new_seek) {
@@ -698,7 +887,6 @@ gst_dvd_read_src_create (GstPushSrc * pushsrc, GstBuffer ** p_buf)
     }
     case GST_DVD_READ_EOS:{
       GST_INFO_OBJECT (src, "Reached EOS");
-      gst_pad_push_event (GST_BASE_SRC (src)->srcpad, gst_event_new_eos ());
       return GST_FLOW_UNEXPECTED;
     }
     case GST_DVD_READ_OK:{
@@ -820,128 +1008,187 @@ gst_dvd_read_src_get_size (GstDvdReadSrc * src, gint64 * size)
 /*** Querying and seeking ***/
 
 static gboolean
-gst_dvd_read_src_do_seek (GstDvdReadSrc * src, GstEvent * event)
+gst_dvd_read_src_handle_seek_event (GstDvdReadSrc * src, GstEvent * event)
 {
   GstSeekFlags flags;
   GstSeekType cur_type, end_type;
-  gint64 new_off, total, cur;
+  gint64 new_off, total;
   GstFormat format;
   GstPad *srcpad;
   gboolean query_ok;
   gdouble rate;
 
-  srcpad = GST_BASE_SRC (src)->srcpad;
-
   gst_event_parse_seek (event, &rate, &format, &flags, &cur_type, &new_off,
       &end_type, NULL);
 
+  if (rate <= 0.0) {
+    GST_DEBUG_OBJECT (src, "cannot do backwards playback yet");
+    return FALSE;
+  }
+
+  if ((flags & GST_SEEK_FLAG_SEGMENT) != 0) {
+    GST_DEBUG_OBJECT (src, "segment seek not supported");
+    return FALSE;
+  }
+
+  if ((flags & GST_SEEK_FLAG_FLUSH) == 0) {
+    GST_DEBUG_OBJECT (src, "can only do flushing seeks at the moment");
+    return FALSE;
+  }
+
   if (end_type != GST_SEEK_TYPE_NONE) {
-    GST_WARNING_OBJECT (src, "End seek type not supported, will be ignored");
+    GST_DEBUG_OBJECT (src, "end seek type not supported");
+    return FALSE;
   }
 
-  switch (format) {
-    case GST_FORMAT_BYTES:
-      break;
-    default:{
-      if (format != chapter_format &&
-          format != sector_format &&
-          format != angle_format && format != title_format) {
-        GST_DEBUG_OBJECT (src, "Unsupported seek format %d (%s)", format,
-            gst_format_get_name (format));
-        return FALSE;
-      }
-      break;
-    }
+  if (cur_type != GST_SEEK_TYPE_SET) {
+    GST_DEBUG_OBJECT (src, "only SEEK_TYPE_SET is supported");
+    return FALSE;
   }
 
-  /* get current offset and length */
-  if (format == GST_FORMAT_BYTES) {
+  if (format == angle_format) {
     GST_OBJECT_LOCK (src);
-    query_ok = gst_dvd_read_src_get_size (src, &total);
-    cur = (gint64) src->cur_pack * DVD_VIDEO_LB_LEN;
-    GST_OBJECT_UNLOCK (src);
-  } else {
-    query_ok = gst_pad_query_duration (srcpad, &format, &total)
-        && gst_pad_query_position (srcpad, &format, &cur);
-  }
-
-  if (!query_ok) {
-    GST_DEBUG_OBJECT (src, "Failed to query duration/position");
-    return FALSE;
-  }
-
-  GST_DEBUG_OBJECT (src, "Current    %s: %12" G_GINT64_FORMAT,
-      gst_format_get_name (format), cur);
-  GST_DEBUG_OBJECT (src, "Total      %s: %12" G_GINT64_FORMAT,
-      gst_format_get_name (format), total);
-
-  /* get absolute */
-  switch (cur_type) {
-    case GST_SEEK_TYPE_SET:
-      /* no-op */
-      break;
-    case GST_SEEK_TYPE_CUR:
-      new_off += cur;
-      break;
-    case GST_SEEK_TYPE_END:
-      new_off = total - new_off;
-      break;
-    default:
-      GST_DEBUG_OBJECT (src, "Unsupported seek method");
+    if (new_off < 0 || new_off >= src->num_angles) {
+      GST_OBJECT_UNLOCK (src);
+      GST_DEBUG_OBJECT (src, "invalid angle %d, only %d available",
+          src->num_angles);
       return FALSE;
-  }
-
-  if (new_off < 0 || new_off >= total) {
-    GST_DEBUG_OBJECT (src, "Invalid seek position %" G_GINT64_FORMAT, new_off);
-    return FALSE;
-  }
-
-  if (cur == new_off) {
-    GST_DEBUG_OBJECT (src, "We're already at that position!");
+    }
+    src->angle = (gint) new_off;
+    GST_OBJECT_UNLOCK (src);
+    GST_DEBUG_OBJECT (src, "switched to angle %d", (gint) new_off + 1);
     return TRUE;
   }
 
-  GST_LOG_OBJECT (src, "Seeking to %s: %12" G_GINT64_FORMAT,
-      gst_format_get_name (format), new_off);
-
-  GST_OBJECT_LOCK (src);
-
-  if (format == angle_format) {
-    src->angle = new_off;
-    goto done;
+  if (format != chapter_format && format != title_format &&
+      format != GST_FORMAT_BYTES && format != GST_FORMAT_TIME) {
+    GST_DEBUG_OBJECT (src, "unsupported seek format %d (%s)", format,
+        gst_format_get_name (format));
+    return FALSE;
   }
 
-  if (format == sector_format) {
-    src->cur_pack = new_off;
-  } else if (format == GST_FORMAT_BYTES) {
-    src->cur_pack = new_off / DVD_VIDEO_LB_LEN;
-    if ((src->cur_pack * DVD_VIDEO_LB_LEN) != new_off) {
-      GST_LOG_OBJECT (src, "rounded down offset %" G_GINT64_FORMAT " => %"
-          G_GINT64_FORMAT, new_off, (gint64) src->cur_pack * DVD_VIDEO_LB_LEN);
+  if (format == GST_FORMAT_BYTES) {
+    GST_DEBUG_OBJECT (src, "Requested seek to byte %" G_GUINT64_FORMAT,
+        new_off);
+  } else if (format == GST_FORMAT_TIME) {
+    GST_DEBUG_OBJECT (src, "Requested seek to time %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (new_off));
+    if (gst_dvd_read_src_get_sector_from_time (src, new_off) < 0) {
+      GST_DEBUG_OBJECT (src, "Can't find sector for requested time");
+      return FALSE;
     }
-  } else if (format == chapter_format) {
-    src->cur_pack = 0;
-    src->chapter = new_off;
-    src->seek_pend_fmt = format;
-  } else if (format == title_format) {
-    src->cur_pack = 0;
-    src->title = new_off;
-    src->chapter = 0;
-    src->seek_pend_fmt = format;
-  } else {
+  }
+
+  srcpad = GST_BASE_SRC_PAD (src);
+
+  /* check whether the seek looks reasonable (ie within possible range) */
+  if (format == GST_FORMAT_BYTES) {
+    GST_OBJECT_LOCK (src);
+    query_ok = gst_dvd_read_src_get_size (src, &total);
     GST_OBJECT_UNLOCK (src);
+  } else {
+    query_ok = gst_pad_query_duration (srcpad, &format, &total);
+  }
+
+  if (!query_ok) {
+    GST_DEBUG_OBJECT (src, "Failed to query duration in format %s",
+        gst_format_get_name (format));
+    return FALSE;
+  }
+
+  GST_DEBUG_OBJECT (src, "Total      %s: %12" G_GINT64_FORMAT,
+      gst_format_get_name (format), total);
+  GST_DEBUG_OBJECT (src, "Seek to    %s: %12" G_GINT64_FORMAT,
+      gst_format_get_name (format), new_off);
+
+  if (new_off >= total) {
+    GST_DEBUG_OBJECT (src, "Seek position out of range");
+    return FALSE;
+  }
+
+  /* set segment to seek format; this allows us to use the do_seek
+   * virtual function and let the base source handle all the tricky
+   * stuff for us. We don't use the segment internally anyway */
+  /* FIXME: can't take the stream lock here - what to do? */
+  GST_OBJECT_LOCK (src);
+  GST_BASE_SRC (src)->segment.format = format;
+  GST_BASE_SRC (src)->segment.start = 0;
+  GST_BASE_SRC (src)->segment.stop = total;
+  GST_BASE_SRC (src)->segment.duration = total;
+  GST_OBJECT_UNLOCK (src);
+
+  return GST_BASE_SRC_CLASS (parent_class)->event (GST_BASE_SRC (src), event);
+}
+
+static gboolean
+gst_dvd_read_src_do_seek (GstBaseSrc * basesrc, GstSegment * s)
+{
+  GstDvdReadSrc *src;
+
+  src = GST_DVD_READ_SRC (basesrc);
+
+  GST_DEBUG_OBJECT (src, "Seeking to %s: %12" G_GINT64_FORMAT,
+      gst_format_get_name (s->format), s->last_stop);
+
+  if (s->format == sector_format || s->format == GST_FORMAT_BYTES
+      || s->format == GST_FORMAT_TIME) {
+    guint old;
+
+    old = src->cur_pack;
+
+    if (s->format == sector_format) {
+      src->cur_pack = s->last_stop;
+    } else if (s->format == GST_FORMAT_TIME) {
+      gint sector;
+
+      sector = gst_dvd_read_src_get_sector_from_time (src, s->last_stop);
+
+      GST_DEBUG_OBJECT (src, "Time %" GST_TIME_FORMAT " => sector %d",
+          GST_TIME_ARGS (s->last_stop), sector);
+
+      /* really shouldn't happen, we've checked this earlier ... */
+      g_return_val_if_fail (sector >= 0, FALSE);
+
+      src->cur_pack = sector;
+    } else {
+      /* byte format */
+      src->cur_pack = s->last_stop / DVD_VIDEO_LB_LEN;
+      if ((src->cur_pack * DVD_VIDEO_LB_LEN) != s->last_stop) {
+        GST_LOG_OBJECT (src, "rounded down offset %" G_GINT64_FORMAT " => %"
+            G_GINT64_FORMAT, s->last_stop,
+            (gint64) src->cur_pack * DVD_VIDEO_LB_LEN);
+      }
+    }
+
+    if (!gst_dvd_read_src_goto_sector (src, src->angle)) {
+      GST_DEBUG_OBJECT (src, "seek to sector 0x%08x failed", src->cur_pack);
+      src->cur_pack = old;
+      return FALSE;
+    }
+
+    GST_LOG_OBJECT (src, "seek to sector 0x%08x ok", src->cur_pack);
+  } else if (s->format == chapter_format) {
+    if (!gst_dvd_read_src_goto_chapter (src, (gint) s->last_stop)) {
+      GST_DEBUG_OBJECT (src, "seek to chapter %d failed",
+          (gint) s->last_stop + 1);
+      return FALSE;
+    }
+    GST_INFO_OBJECT (src, "seek to chapter %d ok", (gint) s->last_stop + 1);
+    src->chapter = s->last_stop;
+  } else if (s->format == title_format) {
+    if (!gst_dvd_read_src_goto_title (src, (gint) s->last_stop, src->angle) ||
+        !gst_dvd_read_src_goto_chapter (src, 0)) {
+      GST_DEBUG_OBJECT (src, "seek to title %d failed", (gint) s->last_stop);
+      return FALSE;
+    }
+    src->title = (gint) s->last_stop;
+    src->chapter = 0;
+    GST_INFO_OBJECT (src, "seek to title %d ok", src->title + 1);
+  } else {
     g_return_val_if_reached (FALSE);
   }
 
-  /* leave for events */
-  src->seek_pend = TRUE;
-  if ((flags & GST_SEEK_FLAG_FLUSH) != 0)
-    src->flush_pend = TRUE;
-
-done:
-
-  GST_OBJECT_UNLOCK (src);
-
+  src->need_newsegment = TRUE;
   return TRUE;
 }
 
@@ -955,7 +1202,7 @@ gst_dvd_read_src_src_event (GstBaseSrc * basesrc, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-      res = gst_dvd_read_src_do_seek (src, event);
+      res = gst_dvd_read_src_handle_seek_event (src, event);
       break;
     default:
       res = GST_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
@@ -1157,26 +1404,29 @@ gst_dvd_read_src_goto_sector (GstDvdReadSrc * src, int angle)
       cur = next;
       if (src->cur_pgc->cell_playback[cur].block_type == BLOCK_TYPE_ANGLE_BLOCK)
         cur += angle;
-      next = gst_dvd_read_src_get_next_cell_for (src, cur);
+      next = gst_dvd_read_src_get_next_cell (src, src->cur_pgc, cur);
     }
   }
 
   GST_DEBUG_OBJECT (src, "Seek to sector %u failed", seek_to);
+
   return FALSE;
 
 done:
-  /* so chapter $chapter and cell $cur contain our sector
-   * of interest. Let's go there! */
-  GST_INFO_OBJECT (src, "Seek succeeded, going to chapter %u, cell %u",
-      chapter, cur);
+  {
+    /* so chapter $chapter and cell $cur contain our sector
+     * of interest. Let's go there! */
+    GST_INFO_OBJECT (src, "Seek succeeded, going to chapter %u, cell %u",
+        chapter + 1, cur);
 
-  gst_dvd_read_src_goto_chapter (src, chapter);
-  src->cur_cell = cur;
-  src->next_cell = next;
-  src->new_cell = FALSE;
-  src->cur_pack = seek_to;
+    gst_dvd_read_src_goto_chapter (src, chapter);
+    src->cur_cell = cur;
+    src->next_cell = next;
+    src->new_cell = FALSE;
+    src->cur_pack = seek_to;
 
-  return TRUE;
+    return TRUE;
+  }
 }
 
 
@@ -1325,6 +1575,12 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (gstgst_dvd_read_src_debug, "dvdreadsrc", 0,
       "DVD reader element based on dvdreadsrc");
 
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   if (!gst_element_register (plugin, "dvdreadsrc", GST_RANK_SECONDARY,
           GST_TYPE_DVD_READ_SRC)) {
     return FALSE;
@@ -1337,4 +1593,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "dvdread",
     "Access a DVD with dvdread",
-    plugin_init, VERSION, "GPL", GST_PACKAGE, GST_ORIGIN)
+    plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
