@@ -243,7 +243,7 @@ gst_id3_tag_get_type (guint type)
 }
 
 /* elementfactory information */
-GstElementDetails gst_id3_tag_details[3] = {
+static const GstElementDetails gst_id3_tag_details[3] = {
   GST_ELEMENT_DETAILS ("id3 tag extractor",
       "Codec/Demuxer/Audio",
       "Extract ID3 tagging information",
@@ -318,8 +318,7 @@ gst_id3_tag_get_caps (GstPad * pad)
     GstCaps *caps = gst_caps_copy (tag->found_caps);
 
     if (CAN_BE_MUXER (tag)) {
-      gst_caps_append (caps,
-          gst_caps_from_string ("application/x-gst-tags; application/x-id3"));
+      gst_caps_append (caps, gst_caps_from_string ("application/x-id3"));
     }
     return caps;
   } else {
@@ -366,6 +365,9 @@ gst_id3_tag_init (GTypeInstance * instance, gpointer g_class)
   if (GST_ID3_TAG_GET_CLASS (tag)->type == GST_ID3_TAG_PARSE_MUX) {
     /* only the muxer class here, all other use sometimes pads */
     gst_id3_tag_add_src_pad (tag);
+    gst_pad_use_fixed_caps (tag->srcpad);
+    gst_pad_set_caps (tag->srcpad,
+        gst_static_pad_template_get_caps (&id3_tag_src_id3_template_factory));
   }
   /* FIXME: for the alli^H^H^H^Hspider - gst_id3_tag_add_src_pad (tag); */
   tag->parse_mode = GST_ID3_TAG_PARSE_BASE;
@@ -859,9 +861,10 @@ gst_id3_tag_get_tag_to_render (GstID3Tag * tag)
   const GstTagList *taglist =
       gst_tag_setter_get_tag_list (GST_TAG_SETTER (tag));
 
-  GST_DEBUG
-      ("preparing taglist to render: event_tags=%p, parsed_tags=%p, taglist=%p",
-      tag->event_tags, tag->parsed_tags, taglist);
+  GST_DEBUG ("preparing taglist to render:");
+  GST_DEBUG (" event_tags  = %" GST_PTR_FORMAT, tag->event_tags);
+  GST_DEBUG (" parsed_tags = %" GST_PTR_FORMAT, tag->parsed_tags);
+  GST_DEBUG (" taglist     = %" GST_PTR_FORMAT, taglist);
 
   if (tag->event_tags)
     ret = gst_tag_list_copy (tag->event_tags);
@@ -873,8 +876,12 @@ gst_id3_tag_get_tag_to_render (GstID3Tag * tag)
   }
   if (taglist) {
     if (ret) {
-      gst_tag_list_insert (ret, taglist,
+      GstTagList *tmp;
+
+      tmp = gst_tag_list_merge (taglist, ret,
           gst_tag_setter_get_tag_merge_mode (GST_TAG_SETTER (tag)));
+      gst_tag_list_free (ret);
+      ret = tmp;
     } else {
       ret = gst_tag_list_copy (taglist);
     }
@@ -1001,6 +1008,7 @@ gst_id3_tag_sink_event (GstPad * pad, GstEvent * event)
             tag_buffer = gst_buffer_new_and_alloc (128);
             if (128 != id3_tag_render (id3, tag_buffer->data))
               g_assert_not_reached ();
+            gst_buffer_set_caps (tag_buffer, GST_PAD_CAPS (tag->srcpad));
             gst_pad_push (tag->srcpad, tag_buffer);
             id3_tag_delete (id3);
           }
@@ -1139,9 +1147,6 @@ gst_id3_tag_src_link (GstPad * pad, GstPad * peer)
   if (strcmp (mimetype, "application/x-id3") == 0) {
     tag->parse_mode = GST_ID3_TAG_PARSE_MUX;
     GST_LOG_OBJECT (tag, "mux operation, using application/x-id3 output");
-  } else if (strcmp (mimetype, "application/x-gst-tags") == 0) {
-    tag->parse_mode = GST_ID3_TAG_PARSE_ANY;
-    GST_LOG_OBJECT (tag, "fast operation, just outputting tags");
   } else {
     tag->parse_mode = GST_ID3_TAG_PARSE_DEMUX;
     GST_LOG_OBJECT (tag, "demux operation, extracting tags");
@@ -1383,6 +1388,7 @@ gst_id3_tag_chain (GstPad * pad, GstBuffer * buffer)
                 id3_tag_render (id3, GST_BUFFER_DATA (tag_buffer));
             g_assert (estimated >= tag->v2tag_size_new);
             GST_BUFFER_SIZE (tag_buffer) = tag->v2tag_size_new;
+            gst_buffer_set_caps (tag_buffer, GST_PAD_CAPS (tag->srcpad));
             gst_pad_push (tag->srcpad, tag_buffer);
             id3_tag_delete (id3);
           }
@@ -1434,6 +1440,7 @@ gst_id3_tag_chain (GstPad * pad, GstBuffer * buffer)
           gst_buffer_unref (buffer);
           buffer = sub;
         }
+        gst_buffer_set_caps (buffer, GST_PAD_CAPS (tag->srcpad));
         gst_pad_push (tag->srcpad, buffer);
       }
       return GST_FLOW_OK;
@@ -1508,13 +1515,12 @@ plugin_init (GstPlugin * plugin)
 
   if (!gst_element_register (plugin, "mad", GST_RANK_SECONDARY,
           gst_mad_get_type ())
-      || !gst_element_register (plugin, "id3mux", GST_RANK_NONE,        /* removed for spider */
+      || !gst_element_register (plugin, "id3mux", GST_RANK_NONE,
           gst_id3_tag_get_type (GST_ID3_TAG_PARSE_MUX))) {
     return FALSE;
   }
 
-  GST_DEBUG_CATEGORY_INIT (gst_id3_tag_debug, "id3tag", 0,
-      "id3 tag reader / setter");
+  GST_DEBUG_CATEGORY_INIT (gst_id3_tag_debug, "id3mux", 0, "id3 tag setter");
 
   return TRUE;
 }
@@ -1523,4 +1529,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "mad",
     "id3 tag manipulation and mp3 decoding based on the mad library",
-    plugin_init, VERSION, "GPL", GST_PACKAGE, GST_ORIGIN)
+    plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
