@@ -29,10 +29,10 @@
 #define GST_MAD(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_MAD,GstMad))
 #define GST_MAD_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_MAD,GstMad))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_MAD,GstMadClass))
 #define GST_IS_MAD(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_MAD))
-#define GST_IS_MAD_CLASS(obj) \
+#define GST_IS_MAD_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MAD))
 
 
@@ -95,28 +95,18 @@ struct _GstMadClass
 };
 
 /* elementfactory information */
-static GstElementDetails gst_mad_details =
+static const GstElementDetails gst_mad_details =
 GST_ELEMENT_DETAILS ("mad mp3 decoder",
     "Codec/Decoder/Audio",
     "Uses mad code to decode mp3 streams",
-    "Wim Taymans <wim.taymans@chello.be>");
+    "Wim Taymans <wim@fluendo.com>");
 
 
-/* Mad signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
 enum
 {
   ARG_0,
   ARG_HALF,
-  ARG_IGNORE_CRC,
-  ARG_METADATA,
-  ARG_STREAMINFO
-      /* FILL ME */
+  ARG_IGNORE_CRC
 };
 
 GST_DEBUG_CATEGORY_STATIC (mad_debug);
@@ -159,10 +149,6 @@ static void gst_mad_get_property (GObject * object, guint prop_id,
 
 static gboolean gst_mad_src_event (GstPad * pad, GstEvent * event);
 
-#if 0
-static const GstFormat *gst_mad_get_formats (GstPad * pad);
-#endif
-
 static const GstQueryType *gst_mad_get_query_types (GstPad * pad);
 
 static gboolean gst_mad_src_query (GstPad * pad, GstQuery * query);
@@ -287,7 +273,7 @@ gst_mad_class_init (GstMadClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->set_property = gst_mad_set_property;
   gobject_class->get_property = gst_mad_get_property;
@@ -334,12 +320,6 @@ gst_mad_init (GstMad * mad)
   gst_pad_set_chain_function (mad->sinkpad, GST_DEBUG_FUNCPTR (gst_mad_chain));
   gst_pad_set_event_function (mad->sinkpad,
       GST_DEBUG_FUNCPTR (gst_mad_sink_event));
-#if 0
-  gst_pad_set_convert_function (mad->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_mad_convert_sink));
-  gst_pad_set_formats_function (mad->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_mad_get_formats));
-#endif
   template = gst_static_pad_template_get (&mad_src_template_factory);
   mad->srcpad = gst_pad_new_from_template (template, "src");
   gst_object_unref (template);
@@ -351,12 +331,6 @@ gst_mad_init (GstMad * mad)
   gst_pad_set_query_type_function (mad->srcpad,
       GST_DEBUG_FUNCPTR (gst_mad_get_query_types));
   gst_pad_use_fixed_caps (mad->srcpad);
-#if 0
-  gst_pad_set_convert_function (mad->srcpad,
-      GST_DEBUG_FUNCPTR (gst_mad_convert_src));
-  gst_pad_set_formats_function (mad->srcpad,
-      GST_DEBUG_FUNCPTR (gst_mad_get_formats));
-#endif
   mad->tempbuffer = g_malloc (MAD_BUFFER_MDLEN * 3);
   mad->tempsize = 0;
   mad->base_byte_offset = 0;
@@ -410,26 +384,6 @@ gst_mad_get_index (GstElement * element)
   return mad->index;
 }
 
-#if 0
-static const GstFormat *
-gst_mad_get_formats (GstPad * pad)
-{
-  static const GstFormat src_formats[] = {
-    GST_FORMAT_BYTES,
-    GST_FORMAT_DEFAULT,
-    GST_FORMAT_TIME,
-    0
-  };
-  static const GstFormat sink_formats[] = {
-    GST_FORMAT_BYTES,
-    GST_FORMAT_TIME,
-    0
-  };
-
-  return (GST_PAD_IS_SRC (pad) ? src_formats : sink_formats);
-}
-#endif
-
 static gboolean
 gst_mad_convert_sink (GstPad * pad, GstFormat src_format, gint64 src_value,
     GstFormat * dest_format, gint64 * dest_value)
@@ -437,6 +391,11 @@ gst_mad_convert_sink (GstPad * pad, GstFormat src_format, gint64 src_value,
   gboolean res = TRUE;
   GstMad *mad;
 
+  if (src_format == *dest_format) {
+    *dest_value = src_value;
+    return TRUE;
+  }
+
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
   if (mad->vbr_average == 0)
@@ -480,6 +439,11 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
   gint bytes_per_sample;
   GstMad *mad;
 
+  if (src_format == *dest_format) {
+    *dest_value = src_value;
+    return TRUE;
+  }
+
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
   bytes_per_sample = MAD_NCHANNELS (&mad->frame.header) * 4;
@@ -498,7 +462,8 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
 
           if (byterate == 0)
             return FALSE;
-          *dest_value = src_value * GST_SECOND / byterate;
+          *dest_value =
+              gst_util_uint64_scale_int (src_value, GST_SECOND, byterate);
           break;
         }
         default:
@@ -513,7 +478,8 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
         case GST_FORMAT_TIME:
           if (mad->frame.header.samplerate == 0)
             return FALSE;
-          *dest_value = src_value * GST_SECOND / mad->frame.header.samplerate;
+          *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
+              mad->frame.header.samplerate);
           break;
         default:
           res = FALSE;
@@ -525,8 +491,8 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           scale = bytes_per_sample;
           /* fallthrough */
         case GST_FORMAT_DEFAULT:
-          *dest_value =
-              src_value * scale * mad->frame.header.samplerate / GST_SECOND;
+          *dest_value = gst_util_uint64_scale_int (src_value,
+              scale * mad->frame.header.samplerate, GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -563,6 +529,10 @@ gst_mad_src_query (GstPad * pad, GstQuery * query)
   peer = gst_pad_get_peer (mad->sinkpad);
 
   switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_FORMATS:
+      gst_query_set_formats (query, 3, GST_FORMAT_DEFAULT, GST_FORMAT_TIME,
+          GST_FORMAT_BYTES);
+      break;
     case GST_QUERY_POSITION:
     {
       GstFormat format;
@@ -599,18 +569,19 @@ gst_mad_src_query (GstPad * pad, GstQuery * query)
     }
     case GST_QUERY_DURATION:
     {
-      GstFormat format;
-      GstFormat rformat;
+      GstFormat bytes_format = GST_FORMAT_BYTES;
+      GstFormat time_format = GST_FORMAT_TIME;
+      GstFormat req_format;
       gint64 total, total_bytes;
 
       /* save requested format */
-      gst_query_parse_duration (query, &format, NULL);
+      gst_query_parse_duration (query, &req_format, NULL);
 
       if (peer == NULL)
         goto error;
 
       /* try any demuxer before us first */
-      if (format == GST_FORMAT_TIME && gst_pad_query (peer, query)) {
+      if (req_format == GST_FORMAT_TIME && gst_pad_query (peer, query)) {
         gst_query_parse_duration (query, NULL, &total);
         GST_LOG_OBJECT (mad, "peer returned duration %" GST_TIME_FORMAT,
             GST_TIME_ARGS (total));
@@ -618,43 +589,35 @@ gst_mad_src_query (GstPad * pad, GstQuery * query)
       }
 
       /* query peer for total length in bytes */
-      gst_query_set_duration (query, GST_FORMAT_BYTES, -1);
-
-      if (!gst_pad_query (peer, query)) {
-        GST_LOG_OBJECT (mad, "query on peer pad failed");
+      if (!gst_pad_query_peer_duration (mad->sinkpad, &bytes_format,
+              &total_bytes) || total_bytes <= 0) {
+        GST_LOG_OBJECT (mad, "duration query on peer pad failed");
         goto error;
       }
 
-      /* get the returned format */
-      gst_query_parse_duration (query, &rformat, &total_bytes);
-      if (rformat == GST_FORMAT_BYTES) {
-        GST_LOG_OBJECT (mad, "peer pad returned total=%lld bytes", total_bytes);
-      } else if (rformat == GST_FORMAT_TIME) {
-        GST_LOG_OBJECT (mad, "peer pad returned total time=%", GST_TIME_FORMAT,
-            GST_TIME_ARGS (total_bytes));
+      GST_LOG_OBJECT (mad, "peer pad returned total=%" G_GINT64_FORMAT
+          " bytes", total_bytes);
+
+      if (!gst_mad_convert_sink (pad, GST_FORMAT_BYTES, total_bytes,
+              &time_format, &total)) {
+        GST_DEBUG_OBJECT (mad, "conversion BYTE => TIME failed");
+        goto error;
+      }
+      if (!gst_mad_convert_src (pad, GST_FORMAT_TIME, total,
+              &req_format, &total)) {
+        GST_DEBUG_OBJECT (mad, "conversion TIME => %s failed",
+            gst_format_get_name (req_format));
+        goto error;
       }
 
-      /* Check if requested format is returned format */
-      if (format == rformat)
-        return TRUE;
+      gst_query_set_duration (query, req_format, total);
 
-      if (total_bytes != -1) {
-        if (format != GST_FORMAT_BYTES) {
-          if (!gst_mad_convert_sink (pad, GST_FORMAT_BYTES, total_bytes,
-                  &format, &total))
-            goto error;
-        } else {
-          total = total_bytes;
-        }
+      if (req_format == GST_FORMAT_TIME) {
+        GST_LOG_OBJECT (mad, "duration=%" GST_TIME_FORMAT,
+            GST_TIME_ARGS (total));
       } else {
-        total = -1;
-      }
-
-      gst_query_set_duration (query, format, total);
-      if (format == GST_FORMAT_TIME) {
-        GST_LOG ("duration=%" GST_TIME_FORMAT, GST_TIME_ARGS (total));
-      } else {
-        GST_LOG ("duration=%" G_GINT64_FORMAT ", format=%u", total, format);
+        GST_LOG_OBJECT (mad, "duration=%" G_GINT64_FORMAT " (%s)",
+            gst_format_get_name (req_format));
       }
       break;
     }
@@ -699,6 +662,7 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   GstSeekFlags flags;
   GstSeekType cur_type, stop_type;
   gint64 cur, stop;
+  GstIndexEntry *entry = NULL;
 
   /* since we know the exact byteoffset of the frame,
      make sure to try bytes first */
@@ -714,9 +678,8 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   gst_event_parse_seek (event, &rate, &format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
-  GstIndexEntry *entry = gst_index_get_assoc_entry (mad->index, mad->index_id,
-      GST_INDEX_LOOKUP_BEFORE, 0,
-      format, cur);
+  entry = gst_index_get_assoc_entry (mad->index, mad->index_id,
+      GST_INDEX_LOOKUP_BEFORE, 0, format, cur);
 
   GST_DEBUG ("index seek");
 
@@ -925,8 +888,6 @@ gst_mad_set_property (GObject * object, guint prop_id,
 {
   GstMad *mad;
 
-  g_return_if_fail (GST_IS_MAD (object));
-
   mad = GST_MAD (object);
 
   switch (prop_id) {
@@ -947,8 +908,6 @@ gst_mad_get_property (GObject * object, guint prop_id,
 {
   GstMad *mad;
 
-  g_return_if_fail (GST_IS_MAD (object));
-
   mad = GST_MAD (object);
 
   switch (prop_id) {
@@ -1132,9 +1091,17 @@ is_xhead (unsigned char *buf)
 #undef LOG
 /*#define LOG*/
 #ifdef LOG
+#ifndef WIN32
 #define lprintf(x...) g_print(x)
 #else
+#define lprintf GST_DEBUG
+#endif
+#else
+#ifndef WIN32
 #define lprintf(x...)
+#else
+#define lprintf GST_DEBUG
+#endif
 #endif
 
 static int
@@ -1540,8 +1507,11 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           mad->total_samples = total;
           mad->last_ts = GST_CLOCK_TIME_NONE;
         }
-        time_offset = mad->total_samples * (GST_SECOND / mad->rate);
-        time_duration = (nsamples * (GST_SECOND / mad->rate));
+        time_offset =
+            gst_util_uint64_scale_int (mad->total_samples, GST_SECOND,
+            mad->rate);
+        time_duration =
+            gst_util_uint64_scale_int (nsamples, GST_SECOND, mad->rate);
       }
 
       if (mad->index) {
@@ -1561,6 +1531,18 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
         gint32 *outdata;
         mad_fixed_t const *left_ch, *right_ch;
 
+        if (mad->need_newsegment) {
+          gint64 start = time_offset;
+
+          GST_DEBUG ("Sending NEWSEGMENT event, start=%" GST_TIME_FORMAT,
+              GST_TIME_ARGS (start));
+
+          gst_pad_push_event (mad->srcpad,
+              gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME,
+                  start, GST_CLOCK_TIME_NONE, start));
+          mad->need_newsegment = FALSE;
+        }
+
         /* will attach the caps to the buffer */
         result =
             gst_pad_alloc_buffer_and_set_caps (mad->srcpad, 0,
@@ -1602,18 +1584,6 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           }
         }
 
-        if (mad->need_newsegment) {
-          gint64 start = GST_BUFFER_TIMESTAMP (outbuffer);
-
-          GST_DEBUG ("Sending NEWSEGMENT event, start=%" GST_TIME_FORMAT,
-              GST_TIME_ARGS (start));
-
-          gst_pad_push_event (mad->srcpad,
-              gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME,
-                  start, GST_CLOCK_TIME_NONE, start));
-          mad->need_newsegment = FALSE;
-        }
-
         result = gst_pad_push (mad->srcpad, outbuffer);
         if (result != GST_FLOW_OK) {
           /* Head for the exit, dropping samples as we go */
