@@ -234,7 +234,7 @@ gst_dvd_demux_class_init (GstDVDDemuxClass * klass)
   GstMPEGParseClass *mpeg_parse_class;
   GstMPEGDemuxClass *mpeg_demux_class;
 
-  parent_class = g_type_class_ref (GST_TYPE_MPEG_DEMUX);
+  parent_class = g_type_class_peek_parent (klass);
 
   gstelement_class = (GstElementClass *) klass;
   mpeg_parse_class = (GstMPEGParseClass *) klass;
@@ -1157,6 +1157,9 @@ static GstStateChangeReturn
 gst_dvd_demux_change_state (GstElement * element, GstStateChange transition)
 {
   GstDVDDemux *dvd_demux = GST_DVD_DEMUX (element);
+  GstStateChangeReturn ret;
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -1169,8 +1172,7 @@ gst_dvd_demux_change_state (GstElement * element, GstStateChange transition)
     default:
       break;
   }
-
-  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  return ret;
 }
 
 gboolean
