@@ -1005,7 +1005,7 @@ gst_mpeg_demux_sync_stream_to_time (GstMPEGDemux * mpeg_demux,
   static gboolean beenhere;     /* FALSE */
 
   if (!beenhere) {
-    g_message ("FIXME: %s\n", __FUNCTION__);
+    g_message ("FIXME: gst_mpeg_demux_sync_stream_to_time\n");
     beenhere = TRUE;
   }
 #if 0
@@ -1214,7 +1214,7 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
   mpeg_demux->video_lock = FALSE;
 
   mpeg_demux->packet_rate_restriction = FALSE;
-  mpeg_demux->total_size_bound = 0LL;
+  mpeg_demux->total_size_bound = G_GINT64_CONSTANT (0);
 
   mpeg_demux->index = NULL;
   mpeg_demux->last_pts = -1;
@@ -1233,6 +1233,9 @@ static GstStateChangeReturn
 gst_mpeg_demux_change_state (GstElement * element, GstStateChange transition)
 {
   GstMPEGDemux *mpeg_demux = GST_MPEG_DEMUX (element);
+  GstStateChangeReturn ret;
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -1241,8 +1244,7 @@ gst_mpeg_demux_change_state (GstElement * element, GstStateChange transition)
     default:
       break;
   }
-
-  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  return ret;
 }
 
 static void
