@@ -321,6 +321,7 @@ gst_mpeg_parse_send_buffer (GstMPEGParse * mpeg_parse, GstBuffer * buffer,
       GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_parse),
           CORE, NEGOTIATION, (NULL), ("failed to set caps"));
       gst_caps_unref (caps);
+      gst_buffer_unref (buffer);
       return GST_FLOW_ERROR;
     }
     gst_caps_unref (caps);
@@ -393,7 +394,7 @@ gst_mpeg_parse_process_event (GstMPEGParse * mpeg_parse, GstEvent * event)
       ret = TRUE;
       break;
     }
-    case GST_EVENT_FLUSH_STOP:
+    case GST_EVENT_FLUSH_STOP:{
       /* Forward the event. */
       if (CLASS (mpeg_parse)->send_event) {
         ret = CLASS (mpeg_parse)->send_event (mpeg_parse, event);
@@ -403,8 +404,9 @@ gst_mpeg_parse_process_event (GstMPEGParse * mpeg_parse, GstEvent * event)
 
       /* Reset the internal fields. */
       gst_mpeg_parse_reset (mpeg_parse);
-
+      gst_mpeg_packetize_flush_cache (mpeg_parse->packetize);
       break;
+    }
     default:
       if (CLASS (mpeg_parse)->send_event) {
         ret = CLASS (mpeg_parse)->send_event (mpeg_parse, event);
@@ -646,15 +648,15 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
     GST_LOG_OBJECT (mpeg_parse,
         "stream current is %1.3fMbs, calculated over %1.3fkB",
         (mpeg_parse->mux_rate * 8) / 1048576.0,
-        mpeg_parse->bytes_since_scr / 1024.0);
+        gst_guint64_to_gdouble (mpeg_parse->bytes_since_scr) / 1024.0);
   }
 
   if (mpeg_parse->avg_bitrate_bytes) {
     GST_LOG_OBJECT (mpeg_parse,
         "stream avg is %1.3fMbs, calculated over %1.3fkB",
-        (float) (mpeg_parse->avg_bitrate_bytes) * 8 * GST_SECOND
-        / mpeg_parse->avg_bitrate_time / 1048576.0,
-        mpeg_parse->avg_bitrate_bytes / 1024.0);
+        gst_guint64_to_gdouble (mpeg_parse->avg_bitrate_bytes) * 8 * GST_SECOND
+        / gst_guint64_to_gdouble (mpeg_parse->avg_bitrate_time) / 1048576.0,
+        gst_guint64_to_gdouble (mpeg_parse->avg_bitrate_bytes) / 1024.0);
   }
 
   /* Range-check the calculated average bitrate. */
@@ -689,12 +691,15 @@ gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
   GstClockTime time;
   guint64 size;
 
-  if (!gst_mpeg_packetize_put (mpeg_parse->packetize, buffer)) {
-    gst_buffer_unref (buffer);
-    goto done;
+  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
+    GST_DEBUG_OBJECT (mpeg_parse, "buffer with DISCONT flag set");
+    gst_mpeg_packetize_flush_cache (mpeg_parse->packetize);
   }
 
-  while (1) {
+  gst_mpeg_packetize_put (mpeg_parse->packetize, buffer);
+  buffer = NULL;
+
+  do {
     result = gst_mpeg_packetize_read (mpeg_parse->packetize, &buffer);
     if (result == GST_FLOW_RESEND) {
       /* there was not enough data in packetizer cache */
@@ -772,8 +777,13 @@ gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
         GST_FLOW_OK);
     time = CLASS (mpeg_parse)->adjust_ts (mpeg_parse,
         MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr));
+
     if (CLASS (mpeg_parse)->send_buffer)
       result = CLASS (mpeg_parse)->send_buffer (mpeg_parse, buffer, time);
+    else
+      gst_buffer_unref (buffer);
+
+    buffer = NULL;
 
     /* Calculate the expected next SCR. */
     if (mpeg_parse->current_scr != MP_INVALID_SCR) {
@@ -809,12 +819,7 @@ gst_mpeg_parse_chain (GstPad * pad, GstBuffer * buffer)
           ", total since SCR: %" G_GINT64_FORMAT ", br: %" G_GINT64_FORMAT
           ", next SCR: %" G_GINT64_FORMAT, size, bss, br, mpeg_parse->next_scr);
     }
-
-    if (result != GST_FLOW_OK && result != GST_FLOW_NOT_LINKED) {
-      gst_buffer_unref (buffer);
-      goto done;
-    }
-  }
+  } while (!GST_FLOW_IS_FATAL (result));
 
 done:
   gst_object_unref (mpeg_parse);
@@ -890,7 +895,8 @@ gst_mpeg_parse_get_rate (GstMPEGParse * mpeg_parse, gint64 * rate)
      * check if we need to update scr_rate
      */
     if ((mpeg_parse->scr_rate == 0) ||
-        (((double) (ABS (mpeg_parse->scr_rate - *rate)) / mpeg_parse->scr_rate)
+        ((gst_guint64_to_gdouble (ABS (mpeg_parse->scr_rate -
+                        *rate)) / gst_guint64_to_gdouble (mpeg_parse->scr_rate))
             >= MP_SCR_RATE_HYST)) {
       mpeg_parse->scr_rate = *rate;
       return TRUE;
@@ -1232,8 +1238,7 @@ gst_mpeg_parse_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       if (!mpeg_parse->packetize) {
         mpeg_parse->packetize =
-            gst_mpeg_packetize_new (mpeg_parse->srcpad,
-            GST_MPEG_PACKETIZE_SYSTEM);
+            gst_mpeg_packetize_new (GST_MPEG_PACKETIZE_SYSTEM);
       }
 
       /* Initialize parser state */
